%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 BFGS1_product.m
%                   Russell Luke
%                   Brian Rife
%               Department of MathSci
%              University of Delaware  
%                  April 12, 2008 
%
%
%
%
% DESCRIPTION:  TO DO
% 
%  Stable BFGS matrix product
%
% INPUT:
%
%     K,mem: dimension variables
%     u_vec, the vector to be multiplied
%     S_mat, the matrix of steps
%     Y_mat, the matrix of gradient differences
%     del,   the initial scaling
%     alpha, a regularization parameter
%
% OUTPUT:
%        
%     v_vec = B*u_vec, where B is the BFGS matrix
%
% USAGE: 
%
%       v_vec = BFGS1_product(K,mem,u_vec, S_mat, Y_mat, del, alpha)
% 
% SUBROUTINE CALLS:
%      none
%      
%
function v_vec = BFGS1_product(K,mem,u_vec, S_mat, Y_mat, del, alpha)

STY_mat = S_mat'*Y_mat;
D_mat   = diag(diag(STY_mat)); %+alpha*eye(size(STY_mat));
L_mat   = tril(STY_mat,-1);

R_mat = 1/del*(S_mat'*S_mat) + L_mat*(diag(diag(STY_mat).^(-1)))*L_mat';
R_mat = chol(R_mat);

Phi_mat   = [Y_mat, (1/del)*S_mat];
Gam1_mat  = [-diag(diag(STY_mat).^(1/2)), diag(diag(STY_mat).^(-1/2))*L_mat'; zeros(size(D_mat)), R_mat];
Gam2_mat  = [diag(diag(STY_mat).^(1/2)), zeros(size(D_mat)); -L_mat*diag(diag(STY_mat).^(-1/2)), R_mat'];

v_vec = Phi_mat'*u_vec;
v_vec = Gam2_mat\v_vec;
v_vec = Gam1_mat\v_vec;
v_vec = (1/del)*u_vec-Phi_mat*v_vec;

     
      
   
