%              RQproc.m
%     Random quadratic Processor
%         written on Feb.6, 2006 by
%             Russell Luke
%         University of  Delaware
%
% DESCRIPTION:  Processor for Random quadratic function for cutting quadratics method for f:R^n->R
%               
%                   F := max_i{f_i := x^T A_i x + 2 b_i^T x}
%
% INPUT:        n = dimension
%               m = number of quadratics.  
%
% OUTPUT:    A = symmetric, positive definite random matrix
%            b=  m vectors of length n with 0\in \ri \conv\{b_i\}
%
% USAGE: [A,b] = RQproc(n,m)
%
% Nonstandard Matlab function calls:  none

function  [A,b] = RQproc(n,m)

% seed=clock;
% seed=seed(6);
seed=13;
A=zeros(n,n*m);
b=zeros(n,m);
for j=1:m-1
		  tmp=Mgen(n,n,10,0,j*seed);
		  A(:,1+n*(j-1):n*j) = tmp'*tmp;
		  b(:,j) = Mgen(n,1,10,0,(j+139)*seed);
end
tmp=Mgen(n,n,10,0,(53)*seed);
A(:,1+n*(m-1):n*m) = tmp'*tmp;
if(m>1)
    b(:,m) = -1/(m-1)*sum(b,2);
else
    b=zeros(n,1);
end


