%                 RQ_auto.m
%         written on March 21, 2008 by
%             Russell Luke
%         University of  Delaware
%
% DESCRIPTION:  Random quadratic function for cutting quadratics method for f:R^n->R
%               
%                   F := max_i(f_i := x^T A_i x + 2 b_i^T x}
%
% INPUT:        x = column vector, length n.  
%               A = array of matrices n-by-n-by-I, variable input
%               b = array of vectors n-by-I, variable input
%
% OUTPUT:    F(x)= max_{j} f_j(x),
%            D(x)= \nabla F(x)
%
% USAGE: [F,[D,H]] = RQ_auto(x, [A, b])
%
% Nonstandard Matlab function calls:  none
function [F,D, H] = RQ_auto(x, A, b)
[n,m]=size(x);

if nargin>1
    [n,I]=size(b);
    tmp  = zeros(I,m);
    for i=1:I
        tmp(i,:) = diag(x'*A(:,1+(i-1)*n:i*n)*x)' + 2*b(:,i)'*x;
    end
    if I>1
        [F,Imax]=max(tmp);
    else
        F=tmp;
        Imax=ones(1,m);
    end
    if(nargout==2)
        D=zeros(n,m);
        if(nargout==3)
            H=zeros(n,n,m);
        end
        for i=1:m
            D(:,i) = 2*A(:,1+(Imax(i)-1)*n:Imax(i)*n)*x(:,i)+2*b(:,Imax(i));
            if(nargout==3)
                H(:,:,i) = 2*A(:,1+(Imax(i)-1)*n:Imax(i)*n);
            end
        end
    end
else
    % generate the same random matrices with ciel(n/2)+1 manifolds where n is
    % the number of rows of x
    I=10; % ceil(n/2)+1;
    A=zeros(n,n*I);
    b=zeros(n,I);
    seed=13;
    for j=1:I-1
        tmp=Mgen(n,n,10,0,j*seed);
        A(:,1+n*(j-1):n*j) = tmp'*tmp;
        b(:,j) = Mgen(n,1,10,0,(j+139)*seed);
    end
    tmp=Mgen(n,n,10,0,(53)*seed);
    A(:,1+n*(I-1):n*I) = tmp'*tmp;
    b(:,I) = -1/(I-1)*sum(b,2);
    % compute max function and gradient, but not the Hessian.
    tmp  = zeros(I,m);
    for i=1:I
        tmp(i,:) = diag(x'*A(:,1+(i-1)*n:i*n)*x)' + 2*b(:,i)'*x;
    end
    if I>1
        [F,Imax]=max(tmp);
    else
        F=tmp;
        Imax=ones(1,m);
    end
    if(nargout==2)
        D=zeros(n,m);
        for i=1:m
            D(:,i) = 2*A(:,1+(Imax(i)-1)*n:Imax(i)*n)*x(:,i)+2*b(:,Imax(i));
        end
    end
end

