%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     TRBFGS1
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 
%
%
% DESCRIPTION:  
%
% INPUT:  
%
% OUTPUT:
%
% USAGE: 
%        d_vec = TRBFGS1(dim,mem,u_vec, ngrad, S_mat, Y_mat, tr, del)
% 
% SUBROUTINE CALLS:

function d_vec = TRBFGS1(dim,mem,u_vec, ngrad, S_mat, Y_mat, tr, del)
% initialization
delinv     = 1/del;
Phi_mat    = [delinv*S_mat,Y_mat];
temp_mat   = Phi_mat';
v0_vec     = temp_mat*u_vec;
temp_mat   = S_mat';
StS_mat    = temp_mat*S_mat;
StY_mat    = temp_mat*Y_mat;
% Changed spdiags to diag
% original line: D_mat = spdiags(spdiags(StY_mat, 0), 0, mem, mem)
D_mat      = diag(diag(StY_mat));
temp_mat   = Y_mat';
YtY_mat    = temp_mat*Y_mat;
temp_mat   = delinv*StY_mat;
temp2_mat  = temp_mat';
Psi_mat    = [delinv^2*StS_mat,temp_mat;temp2_mat,YtY_mat];
temp_mat   = tril(StY_mat,-1);
temp2_mat  = temp_mat';
Gamma_mat  = [delinv*StS_mat,temp_mat;temp2_mat,-D_mat];

% save('mat_vars.mat', 'Phi_mat', 'v0_vec', 'StS_mat', 'StY_mat', ...
%      'D_mat', 'YtY_mat', 'Psi_mat', 'Gamma_mat', 'u_vec')
% disp('--- in TRBFGS1.m ---')
% disp('Psi_mat =')
% disp(Psi_mat)
% disp('Gamma_mat =')
% disp(Gamma_mat)
% disp('--- out TRBFGS1.m ---')

Maxit_TRsub = 12;
% Maxit_TRsub = 0;
Tol_TRsub  = 1e-10;
Iter_TRsub = 0;
dmu        = 1;
mu_max     = ngrad/tr;
mu0        = mu_max/2;
while (Iter_TRsub<=Maxit_TRsub && abs(dmu)>Tol_TRsub)
    Iter_TRsub    = Iter_TRsub+1;
    tau           = delinv+mu0;
    v1_vec        = inv_tGammPhiTPhi_product2(dim,mem,v0_vec,S_mat,Y_mat,delinv,mu0);
    v2_vec        = Gamma_mat*v1_vec;
    v2_vec        = inv_tGammPhiTPhi_product2(dim,mem,v2_vec,S_mat, ...
                                              Y_mat,delinv,mu0);
  
    rminv = delinv;
    % disp('--- in TRBFGS1.m ---')
    % disp(['Iter_TRsub = ', num2str(Iter_TRsub)])
    % disp('v1_vec =')
    % disp(v1_vec)
    % disp('v2_vec =')
    % disp(v2_vec)
    % disp('--- out TRBFGS1.m ---')
    
    temp          = v0_vec'*v1_vec;
    sigma         = v1_vec'*(Psi_mat*v1_vec)+2*temp+ngrad^2;
    % save('mat_vars.mat', 'v1_vec', 'v2_vec', 'v0_vec', 'S_mat', ...
    %      'Y_mat', 'rminv', 'mu0', 'sigma')
    if (sigma<0 || Iter_TRsub == Maxit_TRsub)
        sigma=0;
    else
    end
    sqrtsigma     = sqrt(sigma);
    % Tol_TRsub     = sqrtsigma*Tol0_TRsub;
    temp          = sqrtsigma-tau*tr;
    Delta         = sigma+tau*(v1_vec'*(Psi_mat*v2_vec)+v0_vec'*v2_vec);
    dmu           = (sigma/Delta)*(temp/tr);
    mu1           = mu0+dmu;
    if mu1>=0
        if (mu1>=mu_max)
            if (mu1==mu_max)
                disp('Upper bound on mu is wrong.')
            else
                mu0=mu_max;
            end
        else
            mu0=mu1;
        end
    else
        mu0=.2*mu0;
    end
    % save('mat_vars.mat', 'temp', 'Delta', 'dmu', 'mu1', 'mu0')
end
% save('mat_vars.mat', 'dmu', 'mu0', 'mu_max', 'v1_vec', 'tau')
temp              = Phi_mat*v1_vec;
d_vec             = (u_vec + temp)/tau;
% norm_d                = sqrtsigma/tau;
% pchng             = -(ngrad^2+v0_vec'*v1_vec+mu0*(ns^2)/tau)/(2*tau);  % see TR pg 12'
