%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 invMSB2_product.m
%                   Russell Luke
%               Department of MathSci
%              University of Delaware  
%                  April 12, 2008 
%
%
%
%
% DESCRIPTION:  TO DO
% 
%  Stable inverse-MSB2 matrix product
%
% INPUT:
%
%     K,mem: dimension variables
%     u_vec, the vector to be multiplied
%     S_mat, the matrix of steps
%     Y_mat, the matrix of gradient differences
%     del,   the initial scaling
%     alpha, a regularization parameter
%
% OUTPUT:
%        
%     v_vec = H^{-1}*u_vec, where H^{-1} is the inverse of the MSB2 matrix
%
% USAGE: 
%
%       v_vec = invMSB2_product(K,mem,u_vec, S_mat, Y_mat, del, alpha)
% 
% SUBROUTINE CALLS:
%      none
%      
function v_vec = invMSB2_product(K,mem,u_vec, S_mat, Y_mat, del, alpha)


% gbar=[norm(Y_mat(1:K/2,mem));norm(Y_mat(K/2+1:K,mem))];
% Omega=[gbar(2)/gbar(1)*speye(K/2,K/2), sparse(K/2,K/2);sparse(K/2,K/2),speye(K/2,K/2)];
% Y_mat=Omega*Y_mat;
% S_mat=Omega*S_mat;
% u_vec=Omega*u_vec;
% ny=zeros(mem,1);
% for j=1:mem
%     ny(j)=1/norm(Y_mat(:,j)); 
% end
% Y_mat=Y_mat*diag(ny);
% S_mat=S_mat*diag(ny);

YY_mat=zeros(K,(mem^2+mem)/2);
SS_mat=YY_mat;
M_mat=zeros((mem^2+mem)/2,(mem^2+mem)/2);
YY_mat(:,(mem^2+mem)/2-mem+1:(mem^2+mem)/2)=Y_mat;
YYt_mat=YY_mat';
SS_mat(:,(mem^2+mem)/2-mem+1:(mem^2+mem)/2)=S_mat;
for j=2:mem
    YY_mat(:,((mem-j)^2+mem-j)/2+1:((mem-j+1)^2+mem-j+1)/2)=Y_mat(:,j:mem);
    SS_mat(:,((mem-j)^2+mem-j)/2+1:((mem-j+1)^2+mem-j+1)/2)=S_mat(:,j:mem);
    YYt_mat(((mem-j+1)^2+mem-j+1)/2+1:(mem^2+mem)/2,:) = YY_mat(:,((mem-j+1)^2+mem-j+1)/2+1:(mem^2+mem)/2)';
    M_mat(((mem-j+1)^2+mem-j+1)/2+1:(mem^2+mem)/2,((mem-j)^2+mem-j)/2+1:((mem-j+1)^2+mem-j+1)/2)=...
        -YYt_mat(((mem-j+1)^2+mem-j+1)/2+1:(mem^2+mem)/2,:)*Y_mat(:,j:mem);
end
tmp_mat = eye((mem^2+mem)/2,(mem^2+mem)/2);
% YYt_mat = YY_mat';
tmp_mat = M_mat+(1/del)*YYt_mat*SS_mat+50*alpha*tmp_mat;

% tmp_mat=inv(tmp_mat);
v_vec = (tmp_mat)\((1/del)*YYt_mat*u_vec);
v_vec = (1/del)*u_vec - ((1/del)*SS_mat - YY_mat)*v_vec;
% v_vec=Omega\v_vec;
