from numpy import abs, complex_, exp, load, ones, zeros
from numpy.random import randn, seed

from .fft import FFT, IFFT


def Resize(A, Mnew, Nnew):
    """Resizes an array.

    Takes an array A that is M by N and resizes it to Mnew by Nnew.
    """
    M, N = A.shape
    Anew = zeros((Mnew, Nnew), dtype=A.dtype)
    if Nnew <= N:
        Nstep = int(N/Nnew)
        Mstep = int(M/Mnew)

        countN = -1
        for j in range(0, N, Nstep):
            countN += 1
            countM = -1
            for i in range(0, M, Mstep):
                countM += 1
                Anew[countM, countN] = A[i, j]

    else:
        Nstep = int(Nnew/N)
        Mstep = int(Mnew/M)
        temp = ones((Mstep, Nstep))

        countN = -1
        for j in range(1, N+1):
            countN += 1
            countM = -1
            for i in range(1, M+1):
                countM += 1
                Anew[(i-1)*Mstep:i*Mstep, (j-1)*Nstep:j*Nstep] =\
                    A[countM, countN]*temp

    return Anew


# PYTHON PORT ANNOTATION
# Renamed seed to s, as seed is the name of the function to set the seed.
def MgenN(M, N, v, m, s):
    """Random matrix with normally distributed iid values.

    Random N by M matrix with normally distributed (mean m, variance v)
    iid elements.
    """
    seed(s)
    return v*randn(M, N) + m


def data_processor(newres, noise):
    im = 1j
    data = load('data.npz')
    Xi_A = data['Xi_A']
    temp1 = data['temp1']
    temp2 = data['temp2']

    defocus = (temp1-temp2)/2.
    theta = (temp1+temp2)/2.
    if newres != 512:
        defocus = Resize(defocus, newres, newres)
        theta = Resize(theta, newres, newres)
        Xi_A = Resize(Xi_A, newres, newres)

    aberration = zeros((newres, newres, 3))
    aberration[:, :, 1] = defocus
    aberration[:, :, 2] = -defocus

    diversity = 3
    Y = zeros((newres, newres, 3), dtype=complex_)
    for j in range(diversity):
        Pj = Xi_A*exp(im*(aberration[:, :, j] + theta))
        Y[:, :, j] = IFFT(IFFT(Pj)*FFT(Pj.conj()))

    epsilon = 0
    k = zeros(Y.shape, dtype=complex_)
    for i in range(diversity):
        temp = Y[:, :, i]
        temp = abs(IFFT(temp))
        for j in range(newres):
            for l in range(newres):
                if temp[j, l] < epsilon:
                    temp[j, l] = epsilon
        k[:, :, i] = temp

    if noise:
        k0 = k[:, :, 0] + MgenN(newres, newres, .0625, .125, 3)
        k1 = k[:, :, 1] + MgenN(newres, newres, .0625, .125, 4)
        k2 = k[:, :, 2] + MgenN(newres, newres, .0625, .125, 5)
        for j in range(newres):
            for l in range(newres):
                k0[j, l] = max(k0[j, l], 0)
                k1[j, l] = max(k1[j, l], 0)
                k2[j, l] = max(k2[j, l], 0)
        k[:, :, 0] = k0
        k[:, :, 1] = k1
        k[:, :, 2] = k2

    k = k**.5

    Data_params = {}
    Data_params['k'] = k
    Data_params['aberration'] = aberration
    Data_params['Xi_A'] = Xi_A
    Data_params['diversity'] = diversity

    return Data_params
